/* Copyright (C) 1994 Queen Mary and Westfield College, University of London

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   A copy of the GNU General Public License may be found in
   doc/COPYING in the source tree of this programm.

   This file was written by: Franz Gerschwiler.(goblin@dcs.qmw.ac.uk)

 ****

   modified by: p.hlstede	(peter@calvados.chemie.uni-bremen.de)

   - the code was reorganized and ansified
   - many resource settings were moved to the app-defaults file
   - gloe can now load multiple files

							06.09.1997
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

#include <X11/cursorfont.h>
#include <Xm/Xm.h>
#include <Xm/Text.h>

#include "sys_consts.h"
#include "globals.h"
#include "utils.h"
#include "undo.h"
#include "optionEvents.h"
#include "widgets.h"
#include "fileEvents.h"
#include "dstring.h"

/***  GLOBAL VARIABLES  ***/
XtAppContext app_context;
Widget   toplevel;
char    *FileName = NULL;
Cursor   MainCursor;
Cursor   TextCursor;
Cursor   TimeCursor;
Boolean  AutoSavePossible;

char    *GloePath = "/.gloeGuard";


/***  defined in 'actions.c'  ***/
extern void InstallActions(void);

/* Check to see if Autosave directory exists */
/* There's probably a better way of doing this */
/* Can someone tell me if there is */
static void init_autosave(char *home)
{
   DString      filename;
   struct stat  buf;
   FILE  *fp;

   DStringCompose(&filename, home, GloePath, NULL);
   if (stat(filename.str, &buf) == -1)
      mkdir(filename.str, 0700);

   DStringCat(&filename, "/.testfile");
   fp = fopen(filename.str, "w");
   if (!fp)
   {
      gui_error("Unable to initialize autosave directory/n Autosave facility not available");
      AutoSavePossible = False;
   }
   else
   {
      fclose(fp);
      unlink(filename.str);
      AutoSavePossible = True;
   }
   DStringFree(&filename);
}


void  main(int argc, char **argv)
{
   char  *home;

   XtSetLanguageProc(NULL, NULL, NULL);

   /* INITIALIZE VARIABLES */
   SelectedOnly = False;
   DefaultPrinter = XtMalloc(MAX_PRINTER_LEN);

   /* LOAD CONFIGURATION VALUES */
   setDefaults();
   loadConfig();

   /* CREATE APPLICATION SHELL */
   toplevel = XtVaAppInitialize(&app_context, APP_CLASS,
			NULL, 0, &argc, argv, NULL,
			XmNiconName,       APP_CLASS,
			XmNtitle,          APP_CLASS,
			XmNresizePolicy,   XmRESIZE_GROW,
			XmNdeleteResponse, XmDO_NOTHING,
			NULL);

   InstallActions();

   MainCursor = XCreateFontCursor(XtDisplay(toplevel), XC_hand2);
   TextCursor = XCreateFontCursor(XtDisplay(toplevel), XC_xterm);
   TimeCursor = XCreateFontCursor(XtDisplay(toplevel), XC_watch);

   /* CREATE TOOL WIDGETS */
   gui_createWidgets(toplevel);
   gui_createIcon(toplevel);

   /* REALIZE WIDGETS (this is done before files are loaded */
   /* to assure that the switching of wordwrap works)       */
   XtRealizeWidget(toplevel);

   home = getenv("HOME");
   if (!home) home = "";

   /* LOAD FILE IF GIVEN */

   /* See if the user is trying to recover from a crash */
   if ((argc == 3) && !strcmp(argv[1], "recover"))
   {
      DString  filename;

      DStringCompose(&filename, home, GloePath, "/", argv[2], NULL);
      if (loadfile(filename.str, OPENIT))
      {
	 FileName = Xstrdup(argv[2]);
	 set_status_message("Ok managed to load guard file");
      }
      else
      {
         XmTextSetString(text_canvas, "");
         reset_undo_redo();
	 if (gui_error("Sorry unable to recover") == HELP)
	    gui_info("You may not have specified a valid file name\n"
	             "Or you may not have had autosave turned on");
      }
      DStringFree(&filename);
   }
   else if (argc > 1)
   {
      int       i, flg = 0;
      FileMode  mode = OPENIT;

      FileName = Xstrdup(argv[1]);	/* use fist name as main name */

      XmTextSetString(text_canvas, "");
      reset_undo_redo();

      for (i=1; i<argc; i++)		/* load all files */
      {
	 flg |= loadfile(argv[i], mode);
	 mode = APPENDIT;		/* next files are appended */
      }

      XmTextSetInsertionPosition(text_canvas, 0);
   }

   if (!FileName)
   {
      FileName = Xstrdup("Untitled.gloe");
      XmTextSetString(text_canvas, "");
      reset_undo_redo();
   }

   init_autosave(home);

   set_title_string();
   set_status_message("");

   /* REALIZE WIDGETS */
   XtRealizeWidget(toplevel);

   SetAutoIndentAndWordWrap();
   SetAutoSave();

   set_cursor(toplevel, MainCursor);
   set_cursor(text_canvas, TextCursor);

   XtAppMainLoop(app_context);
}
